@@ -1,7 +1,7 @@
 ##########################################################################
 ##         #   The Coq Proof Assistant / The Coq Development Team       ##
-##  v      #   INRIA, CNRS and contributors - Copyright 1999-2019       ##
-## <O___,, #       (see CREDITS file for the list of authors)           ##
+##  v      #         Copyright INRIA, CNRS and contributors             ##
+## <O___,, # (see version control and CREDITS file for authors & dates) ##
 ##   \VV/  ###############################################################
 ##    //   #    This file is distributed under the terms of the         ##
 ##         #     GNU Lesser General Public License Version 2.1          ##
@@ -13,7 +13,10 @@ Currently geared towards Coq's manual, rather than Coq source files, but one
 could imagine extending it.
 """
 
-# pylint: disable=too-few-public-methods
+# pylint: disable=missing-type-doc, missing-param-doc
+# pylint: disable=missing-return-type-doc, missing-return-doc
+# pylint: disable=too-few-public-methods, too-many-ancestors, arguments-differ
+# pylint: disable=import-outside-toplevel, abstract-method, too-many-lines
 
 import os
 import re
@@ -27,14 +30,14 @@ from docutils.parsers.rst.roles import code_role #, set_classes
 from docutils.parsers.rst.directives.admonitions import BaseAdmonition
 
 from sphinx import addnodes
-from sphinx.roles import XRefRole
-from sphinx.errors import ExtensionError
-from sphinx.util.nodes import set_source_info, set_role_source_info, make_refnode
-from sphinx.util.logging import getLogger, get_node_location
 from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType, Index
-from sphinx.domains.std import token_xrefs
-from sphinx.ext import mathbase
+from sphinx.errors import ExtensionError
+from sphinx.roles import XRefRole
+from sphinx.util.docutils import ReferenceRole
+from sphinx.util.logging import getLogger, get_node_location
+from sphinx.util.nodes import set_source_info, set_role_source_info, make_refnode
+from sphinx.writers.latex import LaTeXTranslator
 
 from . import coqdoc
 from .repl import ansicolors
@@ -43,6 +46,20 @@ from .notations.parsing import ParseError
 from .notations.sphinx import sphinxify
 from .notations.plain import stringify_with_ellipses
 
+# FIXME: Patch this in Sphinx
+# https://github.com/coq/coq/issues/12361
+def visit_desc_signature(self, node):
+    hyper = ''
+    if node.parent['objtype'] != 'describe' and node['ids']:
+        for id in node['ids']:
+            hyper += self.hypertarget(id)
+    self.body.append(hyper)
+    if not node.get('is_multiline'):
+        self._visit_signature_line(node)
+    else:
+        self.body.append('%\n\\pysigstartmultiline\n')
+LaTeXTranslator.visit_desc_signature = visit_desc_signature
+
 PARSE_ERROR = """{}:{} Parse error in notation!
 Offending notation: {}
 Error message: {}"""
@@ -61,6 +78,7 @@ def notation_to_string(notation):
     try:
         return stringify_with_ellipses(notation)
     except ParseError as e:
+        # FIXME source and line aren't defined below — see cc93f419e0
         raise ExtensionError(PARSE_ERROR.format(os.path.basename(source), line, notation, e.msg)) from e
 
 def highlight_using_coqdoc(sentence):
@@ -74,8 +92,7 @@ def make_target(objtype, targetid):
     return "coq:{}.{}".format(objtype, targetid)
 
 def make_math_node(latex, docname, nowrap):
-    node = mathbase.displaymath()
-    node['latex'] = latex
+    node = nodes.math_block(latex, latex)
     node['label'] = None # Otherwise equations are numbered
     node['nowrap'] = nowrap
     node['docname'] = docname
@@ -103,6 +120,10 @@ class CoqObject(ObjectDescription):
     # (eg. “Command”, “Theorem”)
     annotation = None # type: str
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._sig_names = None
+
     def _name_from_signature(self, signature): # pylint: disable=no-self-use, unused-argument
         """Convert a signature into a name to link to.
 
@@ -110,12 +131,15 @@ class CoqObject(ObjectDescription):
         signature”); for example, the signature of the simplest form of the
         ``exact`` tactic is ``exact @id``.
 
-        Returns None by default, in which case no name will be automatically
-        generated.  This is a convenient way to automatically generate names
-        (link targets) without having to write explicit names everywhere.
+        Generates a name for the directive.  Override this method to return None
+        to avoid generating a name automatically.  This is a convenient way
+        to automatically generate names (link targets) without having to write
+        explicit names everywhere.
 
         """
-        return None
+        m = re.match(r"[a-zA-Z0-9_ ]+", signature)
+        if m:
+            return m.group(0).strip()
 
     def _render_signature(self, signature, signode):
         """Render a signature, placing resulting nodes into signode."""
@@ -145,32 +169,34 @@ class CoqObject(ObjectDescription):
         """Prefix signature with the proper annotation, then render it using
         ``_render_signature`` (for example, add “Command” in front of commands).
 
-        :returns: the name given to the resulting node, if any
+        :returns: the names given to the resulting node.
         """
         self._render_annotation(signode)
         self._render_signature(signature, signode)
-        name = self._names.get(signature)
-        if name is None:
+        names = self._sig_names.get(signature)
+        if names is None:
             name = self._name_from_signature(signature) # pylint: disable=assignment-from-none
             # remove trailing ‘.’ found in commands, but not ‘...’ (ellipsis)
             if name is not None and name.endswith(".") and not name.endswith("..."):
                 name = name[:-1]
-        return name
+            names = [name] if name else None
+        return names
 
-    def _warn_if_duplicate_name(self, objects, name):
+    def _warn_if_duplicate_name(self, objects, name, signode):
         """Check that two objects in the same domain don't have the same name."""
         if name in objects:
-            MSG = 'Duplicate object: {}; other is at {}'
-            msg = MSG.format(name, self.env.doc2path(objects[name][0]))
+            MSG = 'Duplicate name {} (other is in {}) attached to {}'
+            msg = MSG.format(name, self.env.doc2path(objects[name][0]), signode)
             self.state_machine.reporter.warning(msg, line=self.lineno)
 
-    def _record_name(self, name, target_id):
-        """Record a name, mapping it to target_id
+    def _record_name(self, name, target_id, signode):
+        """Record a `name` in the current subdomain, mapping it to `target_id`.
 
-        Warns if another object of the same name already exists.
+        Warns if another object of the same name already exists; `signode` is
+        used in the warning.
         """
         names_in_subdomain = self.subdomain_data()
-        self._warn_if_duplicate_name(names_in_subdomain, name)
+        self._warn_if_duplicate_name(names_in_subdomain, name, signode)
         names_in_subdomain[name] = (self.env.docname, self.objtype, target_id)
 
     def _target_id(self, name):
@@ -183,41 +209,50 @@ class CoqObject(ObjectDescription):
             signode['ids'].append(targetid)
             signode['names'].append(name)
             signode['first'] = (not self.names)
-            self.state.document.note_explicit_target(signode)
-            self._record_name(name, targetid)
+            self._record_name(name, targetid, signode)
         return targetid
 
     def _add_index_entry(self, name, target):
         """Add `name` (pointing to `target`) to the main index."""
         assert isinstance(name, str)
-        if not name.startswith("_"):
-            # remove trailing . , found in commands, but not ... (ellipsis)
-            trim = name.endswith(".") and not name.endswith("...")
-            index_text = name[:-1] if trim else name
-            if self.index_suffix:
-                index_text += " " + self.index_suffix
-            self.indexnode['entries'].append(('single', index_text, target, '', None))
+        # remove trailing . , found in commands, but not ... (ellipsis)
+        trim = name.endswith(".") and not name.endswith("...")
+        index_text = name[:-1] if trim else name
+        if self.index_suffix:
+            index_text += " " + self.index_suffix
+        self.indexnode['entries'].append(('single', index_text, target, '', None))
 
-    def add_target_and_index(self, name, _, signode):
-        """Attach a link target to `signode` and an index entry for `name`.
+    def add_target_and_index(self, names, _, signode):
+        """Attach a link target to `signode` and index entries for `names`.
         This is only called (from ``ObjectDescription.run``) if ``:noindex:`` isn't specified."""
-        if name:
-            target = self._add_target(signode, name)
-            self._add_index_entry(name, target)
-            return target
+        if names:
+            for name in names:
+                if isinstance(name, str) and name.startswith('_'):
+                    continue
+                target = self._add_target(signode, name)
+                self._add_index_entry(name, target)
+            self.state.document.note_explicit_target(signode)
 
     def _prepare_names(self):
+        """Construct ``self._sig_names``, a map from signatures to names.
+
+        A node may have either one signature with no name, multiple signatures
+        with one name per signatures, or one signature with multiple names.
+        """
         sigs = self.get_signatures()
         names = self.options.get("name")
         if names is None:
-            self._names = {}
+            self._sig_names = {}
         else:
             names = [n.strip() for n in names.split(";")]
             if len(names) != len(sigs):
-                ERR = ("Expected {} semicolon-separated names, got {}.  " +
-                       "Please provide one name per signature line.")
-                raise self.error(ERR.format(len(names), len(sigs)))
-            self._names = dict(zip(sigs, names))
+                if len(sigs) != 1: #Multiple names for one signature
+                    ERR = ("Expected {} semicolon-separated names, got {}.  " +
+                           "Please provide one name per signature line.")
+                    raise self.error(ERR.format(len(names), len(sigs)))
+                self._sig_names = { sigs[0]: names }
+            else:
+                self._sig_names = { sig: [name] for (sig, name) in zip(sigs, names) }
 
     def run(self):
         self._prepare_names()
@@ -270,7 +305,7 @@ class GallinaObject(PlainObject):
           :math:`p \ge c \rightarrow p \ge \lceil{c}\rceil`.
     """
     subdomain = "thm"
-    index_suffix = "(thm)"
+    index_suffix = "(theorem)"
     annotation = "Theorem"
 
 class VernacObject(NotationObject):
@@ -278,18 +313,17 @@ class VernacObject(NotationObject):
 
     Example::
 
-       .. cmd:: Infix "@symbol" := @term ({+, @modifier}).
+       .. cmd:: Infix @string := @one_term {? ( {+, @syntax_modifier } ) } {? : @ident }
 
           This command is equivalent to :n:`…`.
     """
     subdomain = "cmd"
-    index_suffix = "(cmd)"
+    index_suffix = "(command)"
     annotation = "Command"
 
     def _name_from_signature(self, signature):
         m = re.match(r"[a-zA-Z ]+", signature)
-        if m:
-            return m.group(0).strip()
+        return m.group(0).strip() if m else None
 
 class VernacVariantObject(VernacObject):
     """A variant of a Coq command.
@@ -306,13 +340,13 @@ class VernacVariantObject(VernacObject):
 
              This is equivalent to :n:`Axiom @ident : @term`.
     """
-    index_suffix = "(cmdv)"
+    index_suffix = "(command variant)"
     annotation = "Variant"
 
     def _name_from_signature(self, signature):
         return None
 
-class TacticNotationObject(NotationObject):
+class TacticObject(NotationObject):
     """A tactic, or a tactic notation.
 
     Example::
@@ -322,10 +356,24 @@ class TacticNotationObject(NotationObject):
           :token:`expr` is evaluated to ``v`` which must be a tactic value. …
     """
     subdomain = "tacn"
-    index_suffix = "(tacn)"
-    annotation = None
+    index_suffix = "(tactic)"
+    annotation = "Tactic"
 
-class TacticNotationVariantObject(TacticNotationObject):
+class AttributeObject(NotationObject):
+    """An attribute.
+
+    Example::
+
+       .. attr:: local
+    """
+    subdomain = "attr"
+    index_suffix = "(attribute)"
+    annotation = "Attribute"
+
+    def _name_from_signature(self, signature):
+        return notation_to_string(signature)
+
+class TacticVariantObject(TacticObject):
     """A variant of a tactic.
 
     Example::
@@ -342,9 +390,12 @@ class TacticNotationVariantObject(TacticNotationObject):
              The number is the failure level. If no level is specified, it
              defaults to 0. …
     """
-    index_suffix = "(tacnv)"
+    index_suffix = "(tactic variant)"
     annotation = "Variant"
 
+    def _name_from_signature(self, signature):
+        return None
+
 class OptionObject(NotationObject):
     """A Coq option (a setting with non-boolean value, e.g. a string or numeric value).
 
@@ -357,13 +408,9 @@ class OptionObject(NotationObject):
           application of a tactic.
     """
     subdomain = "opt"
-    index_suffix = "(opt)"
+    index_suffix = "(option)"
     annotation = "Option"
 
-    def _name_from_signature(self, signature):
-        return notation_to_string(signature)
-
-
 class FlagObject(NotationObject):
     """A Coq flag (i.e. a boolean setting).
 
@@ -379,10 +426,6 @@ class FlagObject(NotationObject):
     index_suffix = "(flag)"
     annotation = "Flag"
 
-    def _name_from_signature(self, signature):
-        return notation_to_string(signature)
-
-
 class TableObject(NotationObject):
     """A Coq table, i.e. a setting that is a set of values.
 
@@ -397,77 +440,118 @@ class TableObject(NotationObject):
     index_suffix = "(table)"
     annotation = "Table"
 
-    def _name_from_signature(self, signature):
-        return notation_to_string(signature)
-
 class ProductionObject(CoqObject):
     r"""A grammar production.
 
-    This is useful if you intend to document individual grammar productions.
-    Otherwise, use Sphinx's `production lists
+    Use ``.. prodn`` to document grammar productions instead of Sphinx
+    `production lists
     <http://www.sphinx-doc.org/en/stable/markup/para.html#directive-productionlist>`_.
 
-    Unlike ``.. productionlist``\ s, this directive accepts notation syntax.
-
-
-    Usage::
-
-       .. prodn:: token += production
-       .. prodn:: token ::= production
+    prodn displays multiple productions together with alignment similar to ``.. productionlist``,
+    however unlike ``.. productionlist``\ s, this directive accepts notation syntax.
 
     Example::
 
-        .. prodn:: term += let: @pattern := @term in @term
         .. prodn:: occ_switch ::= { {? {| + | - } } {* @num } }
+        term += let: @pattern := @term in @term
+        | second_production
+
+       The first line defines "occ_switch", which must be unique in the document.  The second
+       references and expands the definition of "term", whose main definition is elsewhere
+       in the document.  The third form is for continuing the
+       definition of a nonterminal when it has multiple productions.  It leaves the first
+       column in the output blank.
 
     """
     subdomain = "prodn"
     #annotation = "Grammar production"
 
+    # handle_signature is called for each line of input in the prodn::
+    # 'signatures' accumulates them in order to combine the lines into a single table:
+    signatures = None # FIXME this should be in init, shouldn't it?
+
     def _render_signature(self, signature, signode):
         raise NotImplementedError(self)
 
     SIG_ERROR = ("{}: Invalid syntax in ``.. prodn::`` directive"
                  + "\nExpected ``name ::= ...`` or ``name += ...``"
-                 + " (e.g. ``pattern += constr:(@ident)``)")
+                 + " (e.g. ``pattern += constr:(@ident)``)\n"
+                 + "  in `{}`")
 
     def handle_signature(self, signature, signode):
-        nsplits = 2
-        parts = signature.split(maxsplit=nsplits)
-        if len(parts) != 3:
-            loc = os.path.basename(get_node_location(signode))
-            raise ExtensionError(ProductionObject.SIG_ERROR.format(loc))
+        parts = signature.split(maxsplit=1)
+        if parts[0].strip() == "|" and len(parts) == 2:
+            lhs = ""
+            op = "|"
+            rhs = parts[1].strip()
+        else:
+            parts = signature.split(maxsplit=2)
+            if len(parts) != 3:
+                loc = os.path.basename(get_node_location(signode))
+                raise ExtensionError(ProductionObject.SIG_ERROR.format(loc, signature))
+            lhs, op, rhs = (part.strip() for part in parts)
+            if op not in ["::=", "+="]:
+                loc = os.path.basename(get_node_location(signode))
+                raise ExtensionError(ProductionObject.SIG_ERROR.format(loc, signature))
 
-        lhs, op, rhs = (part.strip() for part in parts)
-        if op not in ["::=", "+="]:
-            loc = os.path.basename(get_node_location(signode))
-            raise ExtensionError(ProductionObject.SIG_ERROR.format(loc))
-
-        self._render_annotation(signode)
-
-        lhs_op = '{} {} '.format(lhs, op)
-        lhs_node = nodes.literal(lhs_op, lhs_op)
-
-        position = self.state_machine.get_source_and_line(self.lineno)
-        rhs_node = notation_to_sphinx(rhs, *position)
-        signode += addnodes.desc_name(signature, '', lhs_node, rhs_node)
-
-        return ('token', lhs) if op == '::=' else None
+        self.signatures.append((lhs, op, rhs))
+        return [('token', lhs)] if op == '::=' else None
 
     def _add_index_entry(self, name, target):
         pass
 
     def _target_id(self, name):
-        # Use `name[1]` instead of ``nodes.make_id(name[1])`` to work around
-        # https://github.com/sphinx-doc/sphinx/issues/4983
-        return 'grammar-token-{}'.format(name[1])
+        return 'grammar-token-{}'.format(nodes.make_id(name[1]))
 
-    def _record_name(self, name, targetid):
+    def _record_name(self, name, targetid, signode):
         env = self.state.document.settings.env
         objects = env.domaindata['std']['objects']
-        self._warn_if_duplicate_name(objects, name)
+        self._warn_if_duplicate_name(objects, name, signode)
         objects[name] = env.docname, targetid
 
+    def run(self):
+        self.signatures = []
+        indexnode = super().run()[0]  # makes calls to handle_signature
+
+        table = nodes.inline(classes=['prodn-table'])
+        tgroup = nodes.inline(classes=['prodn-column-group'])
+        for _ in range(3):
+            tgroup += nodes.inline(classes=['prodn-column'])
+        table += tgroup
+        tbody = nodes.inline(classes=['prodn-row-group'])
+        table += tbody
+
+        # create rows
+        for signature in self.signatures:
+            lhs, op, rhs = signature
+            position = self.state_machine.get_source_and_line(self.lineno)
+
+            row = nodes.inline(classes=['prodn-row'])
+            entry = nodes.inline(classes=['prodn-cell-nonterminal'])
+            if lhs != "":
+                target_name = 'grammar-token-' + nodes.make_id(lhs)
+                target = nodes.target('', '', ids=[target_name], names=[target_name])
+                # putting prodn-target on the target node won't appear in the tex file
+                inline = nodes.inline(classes=['prodn-target'])
+                inline += target
+                entry += inline
+                entry += notation_to_sphinx('@'+lhs, *position)
+            else:
+                entry += nodes.literal('', '')
+            row += entry
+
+            entry = nodes.inline(classes=['prodn-cell-op'])
+            entry += nodes.literal(op, op)
+            row += entry
+
+            entry = nodes.inline(classes=['prodn-cell-production'])
+            entry += notation_to_sphinx(rhs, *position)
+            row += entry
+
+            tbody += row
+
+        return [indexnode, table] # only this node goes into the doc
+
 class ExceptionObject(NotationObject):
     """An error raised by a Coq command or tactic.
 
@@ -486,7 +570,7 @@ class ExceptionObject(NotationObject):
              Raised if :n:`@tactic` does not fully solve the goal.
     """
     subdomain = "exn"
-    index_suffix = "(err)"
+    index_suffix = "(error)"
     annotation = "Error"
     # Uses “exn” since “err” already is a CSS class added by “writer_aux”.
 
@@ -509,7 +593,7 @@ class WarningObject(NotationObject):
           valid coercion paths are ignored.
     """
     subdomain = "warn"
-    index_suffix = "(warn)"
+    index_suffix = "(warning)"
     annotation = "Warning"
 
     # Generate names automatically
@@ -833,9 +917,33 @@ class CoqtopBlocksTransform(Transform):
         return isinstance(node, nodes.Element) and 'coqtop_options' in node
 
     @staticmethod
-    def split_sentences(source):
-        """Split Coq sentences in source. Could be improved."""
-        return re.split(r"(?<=(?<!\.)\.)\s+", source)
+    def split_lines(source):
+        r"""Split Coq input in chunks
+
+        A chunk is a minimal sequence of consecutive lines of the input that
+        ends with a '.'
+
+        >>> split_lines('A.\nB.''')
+        ['A.', 'B.']
+
+        >>> split_lines('A.\n\nB.''')
+        ['A.', '\nB.']
+
+        >>> split_lines('A.\n\nB.\n''')
+        ['A.', '\nB.']
+
+        >>> split_lines("SearchPattern (_ + _ = _ + _).\n"
+        ...             "SearchPattern (nat -> bool).\n"
+        ...             "SearchPattern (forall l : list _, _ l l).")
+        ... # doctest: +NORMALIZE_WHITESPACE
+        ['SearchPattern (_ + _ = _ + _).',
+         'SearchPattern (nat -> bool).',
+         'SearchPattern (forall l : list _, _ l l).']
+
+        >>> split_lines('SearchHead le.\nSearchHead (@eq bool).')
+        ['SearchHead le.', 'SearchHead (@eq bool).']
+        """
+        return re.split(r"(?<=(?<!\.)\.)\n", source.strip())
 
     @staticmethod
     def parse_options(node):
@@ -862,7 +970,6 @@ class CoqtopBlocksTransform(Transform):
             raise ExtensionError("{}: Exactly one display option must be passed to .. coqtop::".format(loc))
 
         opt_all = 'all' in options
-        opt_none = 'none' in options
         opt_input = 'in' in options
         opt_output = 'out' in options
 
@@ -884,10 +991,7 @@ class CoqtopBlocksTransform(Transform):
 
         :param should_show: Whether this node should be displayed"""
         is_empty = contents is not None and re.match(r"^\s*$", contents)
-        if is_empty or not should_show:
-            return ['coqtop-hidden']
-        else:
-            return []
+        return ['coqtop-hidden'] if is_empty or not should_show else []
 
     @staticmethod
     def make_rawsource(pairs, opt_input, opt_output):
@@ -914,7 +1018,7 @@ class CoqtopBlocksTransform(Transform):
             repl.sendone('Unset Coqtop Exit On Error.')
         if options['warn']:
             repl.sendone('Set Warnings "default".')
-        for sentence in self.split_sentences(node.rawsource):
+        for sentence in self.split_lines(node.rawsource):
             pairs.append((sentence, repl.sendone(sentence)))
         if options['abort']:
             repl.sendone('Abort All.')
@@ -966,7 +1070,7 @@ class CoqtopBlocksTransform(Transform):
                                     if c != 'coqtop-hidden']
 
     @staticmethod
-    def merge_consecutive_coqtop_blocks(app, doctree, _):
+    def merge_consecutive_coqtop_blocks(_app, doctree, _):
         """Merge consecutive divs wrapping lists of Coq sentences; keep ‘dl’s separate."""
         for node in doctree.traverse(CoqtopBlocksTransform.is_coqtop_block):
             if node.parent:
@@ -1018,6 +1122,9 @@ class CoqVernacIndex(CoqSubdomainsIndex):
 class CoqTacticIndex(CoqSubdomainsIndex):
     name, localname, shortname, subdomains = "tacindex", "Tactic Index", "tactics", ["tacn"]
 
+class CoqAttributeIndex(CoqSubdomainsIndex):
+    name, localname, shortname, subdomains = "attrindex", "Attribute Index", "attributes", ["attr"]
+
 class CoqOptionIndex(CoqSubdomainsIndex):
     name, localname, shortname, subdomains = "optindex", "Flags, options and Tables Index", "options", ["flag", "opt", "table"]
 
@@ -1029,7 +1136,7 @@ class CoqExceptionIndex(CoqSubdomainsIndex):
 
 class IndexXRefRole(XRefRole):
     """A link to one of our domain-specific indices."""
-    lowercase = True,
+    lowercase = True
     innernodeclass = nodes.inline
     warn_dangling = True
 
@@ -1040,6 +1147,19 @@ class IndexXRefRole(XRefRole):
                 title = index.localname
         return title, target
 
+class StdGlossaryIndex(Index):
+    name, localname, shortname = "glossindex", "Glossary", "terms"
+
+    def generate(self, docnames=None):
+        content = defaultdict(list)
+
+        for ((type, itemname), (docname, anchor)) in self.domain.data['objects'].items():
+            if type == 'term':
+                entries = content[itemname[0].lower()]
+                entries.append([itemname, 0, docname, anchor, '', '', ''])
+        content = sorted(content.items())
+        return content, False
+
 def GrammarProductionRole(typ, rawtext, text, lineno, inliner, options={}, content=[]):
     """A grammar production not included in a ``productionlist`` directive.
 
@@ -1056,7 +1176,7 @@ def GrammarProductionRole(typ, rawtext, text, lineno, inliner, options={}, conte
     """
     #pylint: disable=dangerous-default-value, unused-argument
     env = inliner.document.settings.env
-    targetid = 'grammar-token-{}'.format(text)
+    targetid = nodes.make_id('grammar-token-{}'.format(text))
     target = nodes.target('', '', ids=[targetid])
     inliner.document.note_explicit_target(target)
     code = nodes.literal(rawtext, text, role=typ.lower())
@@ -1067,6 +1187,44 @@ def GrammarProductionRole(typ, rawtext, text, lineno, inliner, options={}, conte
 
 GrammarProductionRole.role_name = "production"
 
+
+def GlossaryDefRole(typ, rawtext, text, lineno, inliner, options={}, content=[]):
+    """Marks the definition of a glossary term inline in the text.  Matching :term:`XXX`
+    constructs will link to it.  Use the form :gdef:`text <term>` to display "text"
+    for the definition of "term", such as when "term" must be capitalized or plural
+    for grammatical reasons.  The term will also appear in the Glossary Index.
+
+    Examples::
+
+       A :gdef:`prime` number is divisible only by itself and 1.
+       :gdef:`Composite <composite>` numbers are the non-prime numbers.
+    """
+    #pylint: disable=dangerous-default-value, unused-argument
+    env = inliner.document.settings.env
+    std = env.domaindata['std']['objects']
+    m = ReferenceRole.explicit_title_re.match(text)
+    if m:
+        (text, term) = m.groups()
+        text = text.strip()
+    else:
+        term = text
+    key = ('term', term)
+
+    if key in std:
+        MSG = 'Duplicate object: {}; other is at {}'
+        msg = MSG.format(term, env.doc2path(std[key][0]))
+        inliner.document.reporter.warning(msg, line=lineno)
+
+    targetid = nodes.make_id('term-{}'.format(term))
+    std[key] = (env.docname, targetid)
+    target = nodes.target('', '', ids=[targetid], names=[term])
+    inliner.document.note_explicit_target(target)
+    node = nodes.inline(rawtext, '', target, nodes.Text(text), classes=['term-defn'])
+    set_role_source_info(inliner, lineno, node)
+    return [node], []
+
+GlossaryDefRole.role_name = "gdef"
+
 class CoqDomain(Domain):
     """A domain to document Coq code.
 
@@ -1094,6 +1252,7 @@ class CoqDomain(Domain):
         'opt': ObjType('opt', 'opt'),
         'flag': ObjType('flag', 'flag'),
         'table': ObjType('table', 'table'),
+        'attr': ObjType('attr', 'attr'),
         'thm': ObjType('thm', 'thm'),
         'prodn': ObjType('prodn', 'prodn'),
         'exn': ObjType('exn', 'exn'),
@@ -1107,11 +1266,12 @@ class CoqDomain(Domain):
         # the same role.
         'cmd': VernacObject,
         'cmdv': VernacVariantObject,
-        'tacn': TacticNotationObject,
-        'tacv': TacticNotationVariantObject,
+        'tacn': TacticObject,
+        'tacv': TacticVariantObject,
         'opt': OptionObject,
         'flag': FlagObject,
         'table': TableObject,
+        'attr': AttributeObject,
         'thm': GallinaObject,
         'prodn' : ProductionObject,
         'exn': ExceptionObject,
@@ -1125,6 +1285,7 @@ class CoqDomain(Domain):
         'opt': XRefRole(warn_dangling=True),
         'flag': XRefRole(warn_dangling=True),
         'table': XRefRole(warn_dangling=True),
+        'attr': XRefRole(warn_dangling=True),
         'thm': XRefRole(warn_dangling=True),
         'prodn' : XRefRole(warn_dangling=True),
         'exn': XRefRole(warn_dangling=True),
@@ -1136,7 +1297,7 @@ class CoqDomain(Domain):
         'g': CoqCodeRole
     }
 
-    indices = [CoqVernacIndex, CoqTacticIndex, CoqOptionIndex, CoqGallinaIndex, CoqExceptionIndex]
+    indices = [CoqVernacIndex, CoqTacticIndex, CoqOptionIndex, CoqGallinaIndex, CoqExceptionIndex, CoqAttributeIndex]
 
     data_version = 1
     initial_data = {
@@ -1148,6 +1309,7 @@ class CoqDomain(Domain):
             'opt': {},
             'flag': {},
             'table': {},
+            'attr': {},
             'thm': {},
             'prodn' : {},
             'exn': {},
@@ -1160,6 +1322,7 @@ class CoqDomain(Domain):
         for index in CoqDomain.indices:
             if index.name == targetid:
                 return index
+        return None
 
     def get_objects(self):
         # Used for searching and object inventories (intersphinx)
@@ -1191,6 +1354,7 @@ class CoqDomain(Domain):
             if resolved:
                 (todocname, _, targetid) = resolved
                 return make_refnode(builder, fromdocname, todocname, targetid, contnode, targetname)
+        return None
 
     def clear_doc(self, docname_to_clear):
         for subdomain_objects in self.data['objects'].values():
@@ -1225,18 +1389,23 @@ COQ_ADDITIONAL_DIRECTIVES = [CoqtopDirective,
                              InferenceDirective,
                              PreambleDirective]
 
-COQ_ADDITIONAL_ROLES = [GrammarProductionRole]
+COQ_ADDITIONAL_ROLES = [GrammarProductionRole,
+                        GlossaryDefRole]
 
 def setup(app):
     """Register the Coq domain"""
 
     # A few sanity checks:
     subdomains = set(obj.subdomain for obj in CoqDomain.directives.values())
-    assert subdomains.issuperset(chain(*(idx.subdomains for idx in CoqDomain.indices)))
-    assert subdomains.issubset(CoqDomain.roles.keys())
+    found = set (obj for obj in chain(*(idx.subdomains for idx in CoqDomain.indices)))
+    assert subdomains.issuperset(found), "Missing subdomains: {}".format(found.difference(subdomains))
+
+    assert subdomains.issubset(CoqDomain.roles.keys()), \
+        "Missing from CoqDomain.roles: {}".format(subdomains.difference(CoqDomain.roles.keys()))
 
     # Add domain, directives, and roles
     app.add_domain(CoqDomain)
+    app.add_index_to_domain('std', StdGlossaryIndex)
 
     for role in COQ_ADDITIONAL_ROLES:
         app.add_role(role.role_name, role)
