@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -216,27 +216,25 @@ let it_mkLambda_or_LetIn_name env sigma b hyps =
 let get_mangle_names =
   Goptions.declare_bool_option_and_ref
     ~depr:false
-    ~name:"mangle auto-generated names"
     ~key:["Mangle";"Names"]
     ~value:false
 
-let mangle_names_prefix = ref (Id.of_string "_0")
+let mangle_names_prefix =
+  Goptions.declare_interpreted_string_option_and_ref
+    ~depr:false
+    ~key:["Mangle";"Names";"Prefix"]
+    ~value:(Id.of_string "_0")
+    (fun x ->
+      (try
+         Id.of_string x
+       with
+       | CErrors.UserError _ ->
+          CErrors.user_err Pp.(str ("Not a valid identifier: \"" ^ x ^ "\"."))
+      ) |> forget_subscript
+    )
+    (fun x -> Id.to_string x)
 
-let set_prefix x = mangle_names_prefix := forget_subscript x
-
-let () = Goptions.(
-    declare_string_option
-      { optdepr  = false;
-        optname  = "mangled names prefix";
-        optkey   = ["Mangle";"Names";"Prefix"];
-        optread  = (fun () -> Id.to_string !mangle_names_prefix);
-        optwrite = begin fun x ->
-                   set_prefix
-                     (try Id.of_string x
-                      with CErrors.UserError _ -> CErrors.user_err Pp.(str ("Not a valid identifier: \"" ^ x ^ "\".")))
-                   end })
-
-let mangle_id id = if get_mangle_names () then !mangle_names_prefix else id
+let mangle_id id = if get_mangle_names () then mangle_names_prefix () else id
 
 (* Looks for next "good" name by lifting subscript *)
 
@@ -261,15 +259,17 @@ let visible_ids sigma (nenv, c) =
     let (gseen, vseen, ids) = !accu in
     let g = global_of_constr c in
     if not (GlobRef.Set_env.mem g gseen) then
-      begin
-      try
       let gseen = GlobRef.Set_env.add g gseen in
-      let short = Nametab.shortest_qualid_of_global Id.Set.empty g in
-      let dir, id = repr_qualid short in
-      let ids = if DirPath.is_empty dir then Id.Set.add id ids else ids in
+      let ids = match Nametab.shortest_qualid_of_global Id.Set.empty g with
+      | short ->
+        let dir, id = repr_qualid short in
+        if DirPath.is_empty dir then Id.Set.add id ids else ids
+      | exception Not_found ->
+        (* This may happen if given pathological terms or when manipulating
+           open modules *)
+        ids
+      in
       accu := (gseen, vseen, ids)
-      with Not_found when !Flags.in_debugger || !Flags.in_toplevel -> ()
-      end
   | Rel p ->
     let (gseen, vseen, ids) = !accu in
     if p > n && not (Int.Set.mem p vseen) then
