@@ -8,72 +8,108 @@ open Util
 
 (* Functorial interface *)
 
-module type GLexerType = Plexing.Lexer
+type norec
+type mayrec
 
-type ty_norec = TyNoRec
-type ty_mayrec = TyMayRec
+module type S = sig
+  type te
+  type 'c pattern
 
-module type S =
-  sig
-    type te
-    type 'c pattern
-    type parsable
-    val parsable : ?loc:Loc.t -> char Stream.t -> parsable
-    val tokens : string -> (string option * int) list
-    module Entry :
-      sig
-        type 'a e
-        val create : string -> 'a e
-        val parse : 'a e -> parsable -> 'a
-        val name : 'a e -> string
-        val of_parser : string -> (Plexing.location_function -> te Stream.t -> 'a) -> 'a e
-        val parse_token_stream : 'a e -> te Stream.t -> 'a
-        val print : Format.formatter -> 'a e -> unit
-      end
-    type ('self, 'trec, 'a) ty_symbol
-    type ('self, 'trec, 'f, 'r) ty_rule
-    type 'a ty_rules
-    type 'a ty_production
-    val s_nterm : 'a Entry.e -> ('self, ty_norec, 'a) ty_symbol
-    val s_nterml : 'a Entry.e -> string -> ('self, ty_norec, 'a) ty_symbol
-    val s_list0 : ('self, 'trec, 'a) ty_symbol -> ('self, 'trec, 'a list) ty_symbol
-    val s_list0sep :
-      ('self, 'trec, 'a) ty_symbol -> ('self, ty_norec, 'b) ty_symbol -> bool ->
-        ('self, 'trec, 'a list) ty_symbol
-    val s_list1 : ('self, 'trec, 'a) ty_symbol -> ('self, 'trec, 'a list) ty_symbol
-    val s_list1sep :
-      ('self, 'trec, 'a) ty_symbol -> ('self, ty_norec, 'b) ty_symbol -> bool ->
-        ('self, 'trec, 'a list) ty_symbol
-    val s_opt : ('self, 'trec, 'a) ty_symbol -> ('self, 'trec, 'a option) ty_symbol
-    val s_self : ('self, ty_mayrec, 'self) ty_symbol
-    val s_next : ('self, ty_mayrec, 'self) ty_symbol
-    val s_token : 'c pattern -> ('self, ty_norec, 'c) ty_symbol
-    val s_rules : warning:(string -> unit) option -> 'a ty_rules list -> ('self, ty_norec, 'a) ty_symbol
-    val r_stop : ('self, ty_norec, 'r, 'r) ty_rule
-    val r_next :
-      ('self, _, 'a, 'r) ty_rule -> ('self, _, 'b) ty_symbol ->
-        ('self, ty_mayrec, 'b -> 'a, 'r) ty_rule
-    val r_next_norec :
-      ('self, ty_norec, 'a, 'r) ty_rule -> ('self, ty_norec, 'b) ty_symbol ->
-        ('self, ty_norec, 'b -> 'a, 'r) ty_rule
-    val rules : (_, ty_norec, 'f, Loc.t -> 'a) ty_rule * 'f -> 'a ty_rules
-    val production : ('a, _, 'f, Loc.t -> 'a) ty_rule * 'f -> 'a ty_production
-    module Unsafe :
-      sig
-        val clear_entry : 'a Entry.e -> unit
-      end
-    val safe_extend : warning:(string -> unit) option ->
-      'a Entry.e -> Gramext.position option ->
-        (string option * Gramext.g_assoc option * 'a ty_production list)
-          list ->
-        unit
-    val safe_delete_rule : 'a Entry.e -> ('a, _, 'r, 'f) ty_rule -> unit
+  module Parsable : sig
+    type t
+    val make : ?loc:Loc.t -> char Stream.t -> t
+    val comments : t -> ((int * int) * string) list
   end
 
+  val tokens : string -> (string option * int) list
+
+  module Entry : sig
+    type 'a t
+    val make : string -> 'a t
+    val create : string -> 'a t
+    val parse : 'a t -> Parsable.t -> 'a
+    val name : 'a t -> string
+    val of_parser : string -> (Plexing.location_function -> te Stream.t -> 'a) -> 'a t
+    val parse_token_stream : 'a t -> te Stream.t -> 'a
+    val print : Format.formatter -> 'a t -> unit
+  end
+
+  module rec Symbol : sig
+
+    type ('self, 'trec, 'a) t
+    val nterm : 'a Entry.t -> ('self, norec, 'a) t
+    val nterml : 'a Entry.t -> string -> ('self, norec, 'a) t
+    val list0 : ('self, 'trec, 'a) t -> ('self, 'trec, 'a list) t
+    val list0sep :
+      ('self, 'trec, 'a) t -> ('self, norec, 'b) t -> bool ->
+      ('self, 'trec, 'a list) t
+    val list1 : ('self, 'trec, 'a) t -> ('self, 'trec, 'a list) t
+    val list1sep :
+      ('self, 'trec, 'a) t -> ('self, norec, 'b) t -> bool ->
+      ('self, 'trec, 'a list) t
+    val opt : ('self, 'trec, 'a) t -> ('self, 'trec, 'a option) t
+    val self : ('self, mayrec, 'self) t
+    val next : ('self, mayrec, 'self) t
+    val token : 'c pattern -> ('self, norec, 'c) t
+    val rules : 'a Rules.t list -> ('self, norec, 'a) t
+
+  end and Rule : sig
+
+    type ('self, 'trec, 'f, 'r) t
+
+    val stop : ('self, norec, 'r, 'r) t
+    val next :
+      ('self, _, 'a, 'r) t -> ('self, _, 'b) Symbol.t ->
+      ('self, mayrec, 'b -> 'a, 'r) t
+    val next_norec :
+      ('self, norec, 'a, 'r) Rule.t -> ('self, norec, 'b) Symbol.t ->
+      ('self, norec, 'b -> 'a, 'r) t
+
+  end and Rules : sig
+
+    type 'a t
+    val make : (_, norec, 'f, Loc.t -> 'a) Rule.t -> 'f -> 'a t
+
+  end
+
+  module Production : sig
+    type 'a t
+    val make : ('a, _, 'f, Loc.t -> 'a) Rule.t -> 'f -> 'a t
+  end
+
+  type 'a single_extend_statement =
+    string option * Gramext.g_assoc option * 'a Production.t list
+
+  type 'a extend_statement =
+    { pos : Gramext.position option
+    ; data : 'a single_extend_statement list
+    }
+
+  val generalize_symbol : ('a, 'tr, 'c) Symbol.t -> ('a, norec, 'c) Symbol.t option
+
+  val mk_rule : 'a pattern list -> string Rules.t
+
+  (* Used in custom entries, should tweak? *)
+  val level_of_nonterm : ('a, norec, 'c) Symbol.t -> string option
+
+end
+
+module type ExtS = sig
+
+  include S
+
+  val safe_extend : 'a Entry.t -> 'a extend_statement -> unit
+  val safe_delete_rule : 'a Entry.t -> 'a Production.t -> unit
+
+  module Unsafe : sig
+    val clear_entry : 'a Entry.t -> unit
+  end
+
+end
+
 (* Implementation *)
 
-module GMake (L : GLexerType) =
-struct
+module GMake (L : Plexing.S) = struct
 
 type te = L.te
 type 'c pattern = 'c L.pattern
@@ -84,7 +120,7 @@ type grammar =
   { gtokens : (string * string option, int ref) Hashtbl.t }
 
 let egram =
-  {gtokens = Hashtbl.create 301 }
+  { gtokens = Hashtbl.create 301 }
 
 let tokens con =
   let list = ref [] in
@@ -94,12 +130,12 @@ let tokens con =
   !list
 
 type ('a, 'b, 'c) ty_and_rec =
-| NoRec2 : (ty_norec, ty_norec, ty_norec) ty_and_rec
-| MayRec2 : ('a, 'b, ty_mayrec) ty_and_rec
+| NoRec2 : (norec, norec, norec) ty_and_rec
+| MayRec2 : ('a, 'b, mayrec) ty_and_rec
 
 type ('a, 'b, 'c, 'd) ty_and_rec3 =
-| NoRec3 : (ty_norec, ty_norec, ty_norec, ty_norec) ty_and_rec3
-| MayRec3 : ('a, 'b, 'c, ty_mayrec) ty_and_rec3
+| NoRec3 : (norec, norec, norec, norec) ty_and_rec3
+| MayRec3 : ('a, 'b, 'c, mayrec) ty_and_rec3
 
 type 'a ty_entry = {
   ename : string;
@@ -122,26 +158,26 @@ and ('trecs, 'trecp, 'a) ty_rec_level = {
 }
 
 and ('self, 'trec, 'a) ty_symbol =
-| Stoken : 'c pattern -> ('self, ty_norec, 'c) ty_symbol
+| Stoken : 'c pattern -> ('self, norec, 'c) ty_symbol
 | Slist1 : ('self, 'trec, 'a) ty_symbol -> ('self, 'trec, 'a list) ty_symbol
-| Slist1sep : ('self, 'trec, 'a) ty_symbol * ('self, ty_norec, _) ty_symbol * bool -> ('self, 'trec, 'a list) ty_symbol
+| Slist1sep : ('self, 'trec, 'a) ty_symbol * ('self, norec, _) ty_symbol * bool -> ('self, 'trec, 'a list) ty_symbol
 | Slist0 : ('self, 'trec, 'a) ty_symbol -> ('self, 'trec, 'a list) ty_symbol
-| Slist0sep : ('self, 'trec, 'a) ty_symbol * ('self, ty_norec, _) ty_symbol * bool -> ('self, 'trec, 'a list) ty_symbol
+| Slist0sep : ('self, 'trec, 'a) ty_symbol * ('self, norec, _) ty_symbol * bool -> ('self, 'trec, 'a list) ty_symbol
 | Sopt : ('self, 'trec, 'a) ty_symbol -> ('self, 'trec, 'a option) ty_symbol
-| Sself : ('self, ty_mayrec, 'self) ty_symbol
-| Snext : ('self, ty_mayrec, 'self) ty_symbol
-| Snterm : 'a ty_entry -> ('self, ty_norec, 'a) ty_symbol
-| Snterml : 'a ty_entry * string -> ('self, ty_norec, 'a) ty_symbol
+| Sself : ('self, mayrec, 'self) ty_symbol
+| Snext : ('self, mayrec, 'self) ty_symbol
+| Snterm : 'a ty_entry -> ('self, norec, 'a) ty_symbol
+| Snterml : 'a ty_entry * string -> ('self, norec, 'a) ty_symbol
 | Stree : ('self, 'trec, Loc.t -> 'a) ty_tree -> ('self, 'trec, 'a) ty_symbol
 
 and ('self, _, _, 'r) ty_rule =
-| TStop : ('self, ty_norec, 'r, 'r) ty_rule
+| TStop : ('self, norec, 'r, 'r) ty_rule
 | TNext : ('trr, 'trs, 'tr) ty_and_rec * ('self, 'trr, 'a, 'r) ty_rule * ('self, 'trs, 'b) ty_symbol -> ('self, 'tr, 'b -> 'a, 'r) ty_rule
 
 and ('self, 'trec, 'a) ty_tree =
 | Node : ('trn, 'trs, 'trb, 'tr) ty_and_rec3 * ('self, 'trn, 'trs, 'trb, 'b, 'a) ty_node -> ('self, 'tr, 'a) ty_tree
-| LocAct : 'k * 'k list -> ('self, ty_norec, 'k) ty_tree
-| DeadEnd : ('self, ty_norec, 'k) ty_tree
+| LocAct : 'k * 'k list -> ('self, norec, 'k) ty_tree
+| DeadEnd : ('self, norec, 'k) ty_tree
 
 and ('self, 'trec, 'trecs, 'trecb, 'a, 'r) ty_node = {
   node : ('self, 'trec, 'a) ty_symbol;
@@ -150,7 +186,7 @@ and ('self, 'trec, 'trecs, 'trecb, 'a, 'r) ty_node = {
 }
 
 type 'a ty_rules =
-| TRules : (_, ty_norec, 'act, Loc.t -> 'a) ty_rule * 'act -> 'a ty_rules
+| TRules : (_, norec, 'act, Loc.t -> 'a) ty_rule * 'act -> 'a ty_rules
 
 type 'a ty_production =
 | TProd : ('a, _, 'act, Loc.t -> 'a) ty_rule * 'act -> 'a ty_production
@@ -222,13 +258,13 @@ let is_before : type s1 s2 r1 r2 a1 a2. (s1, r1, a1) ty_symbol -> (s2, r2, a2) t
 
 (** Ancillary datatypes *)
 
-type 'a ty_rec = MayRec : ty_mayrec ty_rec | NoRec : ty_norec ty_rec
+type 'a ty_rec = MayRec : mayrec ty_rec | NoRec : norec ty_rec
 
 type ('a, 'b, 'c) ty_and_ex =
-| NR00 : (ty_mayrec, ty_mayrec, ty_mayrec) ty_and_ex
-| NR01 : (ty_mayrec, ty_norec, ty_mayrec) ty_and_ex
-| NR10 : (ty_norec, ty_mayrec, ty_mayrec) ty_and_ex
-| NR11 : (ty_norec, ty_norec, ty_norec) ty_and_ex
+| NR00 : (mayrec, mayrec, mayrec) ty_and_ex
+| NR01 : (mayrec, norec, mayrec) ty_and_ex
+| NR10 : (norec, mayrec, mayrec) ty_and_ex
+| NR11 : (norec, norec, norec) ty_and_ex
 
 type ('a, 'b) ty_mayrec_and_ex =
 | MayRecNR : ('a, 'b, _) ty_and_ex -> ('a, 'b) ty_mayrec_and_ex
@@ -243,7 +279,7 @@ type ('s, 'a, 'r) ty_mayrec_rule =
 | MayRecRule : ('s, _, 'a, 'r) ty_rule -> ('s, 'a, 'r) ty_mayrec_rule
 
 type ('self, 'trec, _) ty_symbols =
-| TNil : ('self, ty_norec, unit) ty_symbols
+| TNil : ('self, norec, unit) ty_symbols
 | TCns : ('trh, 'trt, 'tr) ty_and_rec * ('self, 'trh, 'a) ty_symbol * ('self, 'trt, 'b) ty_symbols -> ('self, 'tr, 'a * 'b) ty_symbols
 
 (** ('i, 'p, 'f, 'r) rel_prod0 ~
@@ -306,29 +342,29 @@ let and_and_tree (type s tr' trt tr trn trs trb f) (ar : (tr', trt, tr) ty_and_r
   | MayRec2, _, MayRec -> MayRec2 | MayRec2, _, NoRec -> MayRec2
   | NoRec2, NoRec3, NoRec -> NoRec2
 
-let insert_tree (type s trs trt tr p k a) ~warning entry_name (ar : (trs, trt, tr) ty_and_ex) (gsymbols : (s, trs, p) ty_symbols) (pf : (p, k, a) rel_prod) (action : k) (tree : (s, trt, a) ty_tree) : (s, tr, a) ty_tree =
+let insert_tree (type s trs trt tr p k a) entry_name (ar : (trs, trt, tr) ty_and_ex) (gsymbols : (s, trs, p) ty_symbols) (pf : (p, k, a) rel_prod) (action : k) (tree : (s, trt, a) ty_tree) : (s, tr, a) ty_tree =
   let rec insert : type trs trt tr p f k. (trs, trt, tr) ty_and_ex -> (s, trs, p) ty_symbols -> (p, k, f) rel_prod -> (s, trt, f) ty_tree -> k -> (s, tr, f) ty_tree  =
     fun ar symbols pf tree action ->
     match symbols, pf with
       TCns (ars, s, sl), RelS pf -> insert_in_tree ar ars s sl pf tree action
     | TNil, Rel0 ->
         let node (type tb) ({node = s; son = son; brother = bro} : (_, _, _, tb, _, _) ty_node) =
-          let ar : (ty_norec, tb, tb) ty_and_ex =
+          let ar : (norec, tb, tb) ty_and_ex =
             match get_rec_tree bro with MayRec -> NR10 | NoRec -> NR11 in
           {node = s; son = son; brother = insert ar TNil Rel0 bro action} in
         match ar, tree with
         | NR10, Node (_, n) -> Node (MayRec3, node n)
         | NR11, Node (NoRec3, n) -> Node (NoRec3, node n)
         | NR11, LocAct (old_action, action_list) ->
-          begin match warning with
-            | None -> ()
-            | Some warn_fn ->
+          (* What to do about this warning? For now it is disabled *)
+          if false then
+            begin
               let msg =
                 "<W> Grammar extension: " ^
                 (if entry_name = "" then "" else "in ["^entry_name^"%s], ") ^
                 "some rule has been masked" in
-              warn_fn msg
-          end;
+              Feedback.msg_warning (Pp.str msg)
+            end;
           LocAct (action, old_action :: action_list)
         | NR11, DeadEnd -> LocAct (action, [])
   and insert_in_tree : type trs trs' trs'' trt tr a p f k. (trs'', trt, tr) ty_and_ex -> (trs, trs', trs'') ty_and_rec -> (s, trs, a) ty_symbol -> (s, trs', p) ty_symbols -> (p, k, a -> f) rel_prod -> (s, trt, f) ty_tree -> k -> (s, tr, f) ty_tree =
@@ -387,21 +423,21 @@ let insert_tree (type s trs trt tr p k a) ~warning entry_name (ar : (trs, trt, t
   in
   insert ar gsymbols pf tree action
 
-let insert_tree_norec (type s p k a) ~warning entry_name (gsymbols : (s, ty_norec, p) ty_symbols) (pf : (p, k, a) rel_prod) (action : k) (tree : (s, ty_norec, a) ty_tree) : (s, ty_norec, a) ty_tree =
-  insert_tree ~warning entry_name NR11 gsymbols pf action tree
+let insert_tree_norec (type s p k a) entry_name (gsymbols : (s, norec, p) ty_symbols) (pf : (p, k, a) rel_prod) (action : k) (tree : (s, norec, a) ty_tree) : (s, norec, a) ty_tree =
+  insert_tree entry_name NR11 gsymbols pf action tree
 
-let insert_tree (type s trs trt p k a) ~warning entry_name (gsymbols : (s, trs, p) ty_symbols) (pf : (p, k, a) rel_prod) (action : k) (tree : (s, trt, a) ty_tree) : (s, a) ty_mayrec_tree =
+let insert_tree (type s trs trt p k a) entry_name (gsymbols : (s, trs, p) ty_symbols) (pf : (p, k, a) rel_prod) (action : k) (tree : (s, trt, a) ty_tree) : (s, a) ty_mayrec_tree =
   let MayRecNR ar = and_symbols_tree gsymbols tree in
-  MayRecTree (insert_tree ~warning entry_name ar gsymbols pf action tree)
+  MayRecTree (insert_tree entry_name ar gsymbols pf action tree)
 
-let srules (type self a) ~warning (rl : a ty_rules list) : (self, ty_norec, a) ty_symbol =
-  let rec retype_tree : type s a. (s, ty_norec, a) ty_tree -> (self, ty_norec, a) ty_tree =
+let srules (type self a) (rl : a ty_rules list) : (self, norec, a) ty_symbol =
+  let rec retype_tree : type s a. (s, norec, a) ty_tree -> (self, norec, a) ty_tree =
     function
     | Node (NoRec3, {node = s; son = son; brother = bro}) ->
       Node (NoRec3, {node = retype_symbol s; son = retype_tree son; brother = retype_tree bro})
     | LocAct (k, kl) -> LocAct (k, kl)
     | DeadEnd -> DeadEnd
-  and retype_symbol : type s a. (s, ty_norec, a) ty_symbol -> (self, ty_norec, a) ty_symbol =
+  and retype_symbol : type s a. (s, norec, a) ty_symbol -> (self, norec, a) ty_symbol =
     function
     | Stoken p -> Stoken p
     | Slist1 s -> Slist1 (retype_symbol s)
@@ -412,7 +448,7 @@ let srules (type self a) ~warning (rl : a ty_rules list) : (self, ty_norec, a) t
     | Snterm e -> Snterm e
     | Snterml (e, l) -> Snterml (e, l)
     | Stree t -> Stree (retype_tree t) in
-  let rec retype_rule : type s k r. (s, ty_norec, k, r) ty_rule -> (self, ty_norec, k, r) ty_rule =
+  let rec retype_rule : type s k r. (s, norec, k, r) ty_rule -> (self, norec, k, r) ty_rule =
     function
     | TStop -> TStop
     | TNext (NoRec2, r, s) -> TNext (NoRec2, retype_rule r, retype_symbol s) in
@@ -421,7 +457,7 @@ let srules (type self a) ~warning (rl : a ty_rules list) : (self, ty_norec, a) t
       (fun tree (TRules (symbols, action)) ->
         let symbols = retype_rule symbols in
         let AnyS (symbols, pf) = get_symbols symbols in
-        insert_tree_norec ~warning "" symbols pf action tree)
+        insert_tree_norec "" symbols pf action tree)
       DeadEnd rl
   in
   Stree t
@@ -431,19 +467,19 @@ let is_level_labelled n (Level lev) =
     Some n1 -> n = n1
   | None -> false
 
-let insert_level (type s tr p k) ~warning entry_name (symbols : (s, tr, p) ty_symbols) (pf : (p, k, Loc.t -> s) rel_prod) (action : k) (slev : s ty_level) : s ty_level =
+let insert_level (type s tr p k) entry_name (symbols : (s, tr, p) ty_symbols) (pf : (p, k, Loc.t -> s) rel_prod) (action : k) (slev : s ty_level) : s ty_level =
   match symbols with
   | TCns (_, Sself, symbols) ->
       let Level slev = slev in
       let RelS pf = pf in
-      let MayRecTree lsuffix = insert_tree ~warning entry_name symbols pf action slev.lsuffix in
+      let MayRecTree lsuffix = insert_tree entry_name symbols pf action slev.lsuffix in
       Level
       {assoc = slev.assoc; lname = slev.lname;
        lsuffix = lsuffix;
        lprefix = slev.lprefix}
   | _ ->
       let Level slev = slev in
-      let MayRecTree lprefix = insert_tree ~warning entry_name symbols pf action slev.lprefix in
+      let MayRecTree lprefix = insert_tree entry_name symbols pf action slev.lprefix in
       Level
       {assoc = slev.assoc; lname = slev.lname; lsuffix = slev.lsuffix;
        lprefix = lprefix}
@@ -457,34 +493,27 @@ let empty_lev lname assoc =
   Level
   {assoc = assoc; lname = lname; lsuffix = DeadEnd; lprefix = DeadEnd}
 
-let change_lev ~warning (Level lev) n lname assoc =
+let change_lev (Level lev) n lname assoc =
   let a =
     match assoc with
       None -> lev.assoc
     | Some a ->
       if a <> lev.assoc then
-        begin
-          match warning with
-          | None -> ()
-          | Some warn_fn ->
-            warn_fn ("<W> Changing associativity of level \""^n^"\"")
-        end;
-        a
+        Feedback.msg_warning (Pp.str ("<W> Changing associativity of level \""^n^"\""));
+      a
   in
-  begin match lname with
-    Some n ->
-      if lname <> lev.lname then
-        begin match warning with
-          | None -> ()
-          | Some warn_fn ->
-            warn_fn ("<W> Level label \""^n^"\" ignored")
-        end;
-  | None -> ()
+  begin
+    match lname with
+    | Some n ->
+      (* warning disabled; it was in the past *)
+      if false && lname <> lev.lname then
+        Feedback.msg_warning (Pp.str ("<W> Level label \""^n^"\" ignored"))
+    | None -> ()
   end;
   Level
   {assoc = a; lname = lev.lname; lsuffix = lev.lsuffix; lprefix = lev.lprefix}
 
-let get_level ~warning entry position levs =
+let get_level entry position levs =
   match position with
     Some First -> [], empty_lev, levs
   | Some Last -> levs, empty_lev, []
@@ -497,7 +526,7 @@ let get_level ~warning entry position levs =
             flush stderr;
             failwith "Grammar.extend"
         | lev :: levs ->
-            if is_level_labelled n lev then [], change_lev ~warning lev n, levs
+            if is_level_labelled n lev then [], change_lev lev n, levs
             else
               let (levs1, rlev, levs2) = get levs in lev :: levs1, rlev, levs2
       in
@@ -532,7 +561,7 @@ let get_level ~warning entry position levs =
       get levs
   | None ->
       match levs with
-        lev :: levs -> [], change_lev ~warning lev "<top>", levs
+        lev :: levs -> [], change_lev lev "<top>", levs
       | [] -> [], empty_lev, []
 
 let change_to_self0 (type s) (type trec) (type a) (entry : s ty_entry) : (s, trec, a) ty_symbol -> (s, a) ty_mayrec_symbol =
@@ -582,7 +611,7 @@ let insert_tokens gram symbols =
   in
   linsert symbols
 
-let levels_of_rules ~warning entry position rules =
+let levels_of_rules entry position rules =
   let elev =
     match entry.edesc with
       Dlevels elev -> elev
@@ -594,7 +623,7 @@ let levels_of_rules ~warning entry position rules =
   match rules with
   | [] -> elev
   | _ ->
-    let (levs1, make_lev, levs2) = get_level ~warning entry position elev in
+    let (levs1, make_lev, levs2) = get_level entry position elev in
     let (levs, _) =
       List.fold_left
         (fun (levs, make_lev) (lname, assoc, level) ->
@@ -605,7 +634,7 @@ let levels_of_rules ~warning entry position rules =
                   let MayRecRule symbols = change_to_self entry symbols in
                   let AnyS (symbols, pf) = get_symbols symbols in
                   insert_tokens egram symbols;
-                  insert_level ~warning entry.ename symbols pf action lev)
+                  insert_level entry.ename symbols pf action lev)
                lev level
            in
            lev :: levs, empty_lev)
@@ -831,8 +860,9 @@ let rec print_symbol : type s tr r. formatter -> (s, tr, r) ty_symbol -> unit =
       fprintf ppf "LIST1 %a SEP %a%s" print_symbol1 s print_symbol1 t
         (if osep then " OPT_SEP" else "")
   | Sopt s -> fprintf ppf "OPT %a" print_symbol1 s
-  | Stoken p when L.tok_pattern_strings p <> ("", None) ->
+  | Stoken p ->
      begin match L.tok_pattern_strings p with
+     | "", Some s -> print_str ppf s
      | con, Some prm -> fprintf ppf "%s@ %a" con print_str prm
      | con, None -> fprintf ppf "%s" con end
   | Snterml (e, l) ->
@@ -1036,7 +1066,7 @@ let level_number entry lab =
     Dlevels elev -> lookup 0 elev
   | Dparser _ -> raise Not_found
 
-let rec top_symb : type s tr a. s ty_entry -> (s, tr, a) ty_symbol -> (s, ty_norec, a) ty_symbol =
+let rec top_symb : type s tr a. s ty_entry -> (s, tr, a) ty_symbol -> (s, norec, a) ty_symbol =
   fun entry ->
   function
     Sself -> Snterm entry
@@ -1460,8 +1490,8 @@ let init_entry_functions entry =
        let f = continue_parser_of_entry entry in
        entry.econtinue <- f; f lev bp a strm)
 
-let extend_entry ~warning entry position rules =
-    let elev = levels_of_rules ~warning entry position rules in
+let extend_entry entry position rules =
+    let elev = levels_of_rules entry position rules in
     entry.edesc <- Dlevels elev; init_entry_functions entry
 
 (* Deleting a rule *)
@@ -1483,105 +1513,259 @@ let delete_rule entry sl =
 
 (* Normal interface *)
 
-type parsable =
-  { pa_chr_strm : char Stream.t;
-    pa_tok_strm : L.te Stream.t;
-    pa_loc_func : Plexing.location_function }
+module Parsable = struct
 
-let parse_parsable entry p =
-  let efun = entry.estart 0 in
-  let ts = p.pa_tok_strm in
-  let cs = p.pa_chr_strm in
-  let fun_loc = p.pa_loc_func in
-  let restore =
-    let old_floc = !floc in
-    let old_tc = !token_count in
-    fun () -> floc := old_floc; token_count := old_tc
-  in
-  let get_loc () =
-    try
-      let cnt = Stream.count ts in
-      (* Ensure that the token at location cnt has been peeked so that
-         the location function knows about it *)
-      let _ = Stream.peek ts in
-      let loc = fun_loc cnt in
-      if !token_count - 1 <= cnt then loc
-      else Loc.merge loc (fun_loc (!token_count - 1))
-    with Failure _ -> Ploc.make_unlined (Stream.count cs, Stream.count cs + 1)
-  in
-  floc := fun_loc;
-  token_count := 0;
-  try let r = efun ts in restore (); r with
-    Stream.Failure ->
+  type t =
+    { pa_chr_strm : char Stream.t
+    ; pa_tok_strm : L.te Stream.t
+    ; pa_loc_func : Plexing.location_function
+    ; lexer_state : L.State.t ref }
+
+  let parse_parsable entry p =
+    let efun = entry.estart 0 in
+    let ts = p.pa_tok_strm in
+    let cs = p.pa_chr_strm in
+    let fun_loc = p.pa_loc_func in
+    let restore =
+      let old_floc = !floc in
+      let old_tc = !token_count in
+      fun () -> floc := old_floc; token_count := old_tc
+    in
+    let get_loc () =
+      try
+        let cnt = Stream.count ts in
+        (* Ensure that the token at location cnt has been peeked so that
+           the location function knows about it *)
+        let _ = Stream.peek ts in
+        let loc = fun_loc cnt in
+        if !token_count - 1 <= cnt then loc
+        else Loc.merge loc (fun_loc (!token_count - 1))
+      with Failure _ -> Ploc.make_unlined (Stream.count cs, Stream.count cs + 1)
+    in
+    floc := fun_loc;
+    token_count := 0;
+    try let r = efun ts in restore (); r with
+      Stream.Failure ->
       let loc = get_loc () in
       restore ();
       Ploc.raise loc (Stream.Error ("illegal begin of " ^ entry.ename))
-  | Stream.Error _ as exc ->
+    | Stream.Error _ as exc ->
       let loc = get_loc () in restore (); Ploc.raise loc exc
-  | exc ->
+    | exc ->
       let loc = Stream.count cs, Stream.count cs + 1 in
       restore (); Ploc.raise (Ploc.make_unlined loc) exc
 
-(* Unsafe *)
+  let parse_parsable e p =
+    L.State.set !(p.lexer_state);
+    try
+      let c = parse_parsable e p in
+      p.lexer_state := L.State.get ();
+      c
+    with Ploc.Exc (loc,e) ->
+      L.State.drop ();
+      let loc' = Loc.get_loc (Exninfo.info e) in
+      let loc = match loc' with None -> loc | Some loc -> loc in
+      Loc.raise ~loc e
 
-let clear_entry e =
-  e.estart <- (fun _ (strm__ : _ Stream.t) -> raise Stream.Failure);
-  e.econtinue <- (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
-  match e.edesc with
-    Dlevels _ -> e.edesc <- Dlevels []
-  | Dparser _ -> ()
+  let make ?loc cs =
+    let lexer_state = ref (L.State.init ()) in
+    L.State.set !lexer_state;
+    let (ts, lf) = L.tok_func ?loc cs in
+    lexer_state := L.State.get ();
+    {pa_chr_strm = cs; pa_tok_strm = ts; pa_loc_func = lf; lexer_state}
 
-    let parsable ?loc cs =
-      let (ts, lf) = L.tok_func ?loc cs in
-      {pa_chr_strm = cs; pa_tok_strm = ts; pa_loc_func = lf}
-    module Entry =
-      struct
-        type 'a e = 'a ty_entry
-        let create n =
-          { ename = n; estart = empty_entry n;
-           econtinue =
-             (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
-           edesc = Dlevels []}
-        let parse (e : 'a e) p : 'a =
-          parse_parsable e p
-        let parse_token_stream (e : 'a e) ts : 'a =
-          e.estart 0 ts
-        let name e = e.ename
-        let of_parser n (p : Plexing.location_function -> te Stream.t -> 'a) : 'a e =
-          { ename = n;
-           estart = (fun _ -> p !floc);
-           econtinue =
-             (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
-           edesc = Dparser p}
-        let print ppf e = fprintf ppf "%a@." print_entry e
-      end
-    let s_nterm e = Snterm e
-    let s_nterml e l = Snterml (e, l)
-    let s_list0 s = Slist0 s
-    let s_list0sep s sep b = Slist0sep (s, sep, b)
-    let s_list1 s = Slist1 s
-    let s_list1sep s sep b = Slist1sep (s, sep, b)
-    let s_opt s = Sopt s
-    let s_self = Sself
-    let s_next = Snext
-    let s_token tok = Stoken tok
-    let s_rules ~warning (t : 'a ty_rules list) = srules ~warning t
-    let r_stop = TStop
-    let r_next r s = TNext (MayRec2, r, s)
-    let r_next_norec r s = TNext (NoRec2, r, s)
-    let rules (p, act) = TRules (p, act)
-    let production (p, act) = TProd (p, act)
-    module Unsafe =
-      struct
-        let clear_entry = clear_entry
-      end
-    let safe_extend ~warning (e : 'a Entry.e) pos
-        (r :
-         (string option * Gramext.g_assoc option * 'a ty_production list)
-           list) =
-      extend_entry ~warning e pos r
-    let safe_delete_rule e r =
-      let AnyS (symbols, _) = get_symbols r in
-      delete_rule e symbols
+  let comments p = L.State.get_comments !(p.lexer_state)
+
+end
+
+module Entry = struct
+  type 'a t = 'a ty_entry
+  let make n =
+    { ename = n; estart = empty_entry n;
+      econtinue =
+        (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
+      edesc = Dlevels []}
+  let create = make
+  let parse (e : 'a t) p : 'a =
+    Parsable.parse_parsable e p
+  let parse_token_stream (e : 'a t) ts : 'a =
+    e.estart 0 ts
+  let name e = e.ename
+  let of_parser n (p : Plexing.location_function -> te Stream.t -> 'a) : 'a t =
+    { ename = n;
+      estart = (fun _ -> p !floc);
+      econtinue =
+        (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
+      edesc = Dparser p}
+  let print ppf e = fprintf ppf "%a@." print_entry e
+end
+
+module rec Symbol : sig
+
+  type ('self, 'trec, 'a) t = ('self, 'trec, 'a) ty_symbol
+
+  val nterm : 'a Entry.t -> ('self, norec, 'a) t
+  val nterml : 'a Entry.t -> string -> ('self, norec, 'a) t
+  val list0 : ('self, 'trec, 'a) t -> ('self, 'trec, 'a list) t
+  val list0sep :
+    ('self, 'trec, 'a) t -> ('self, norec, 'b) t -> bool ->
+    ('self, 'trec, 'a list) t
+  val list1 : ('self, 'trec, 'a) t -> ('self, 'trec, 'a list) t
+  val list1sep :
+    ('self, 'trec, 'a) t -> ('self, norec, 'b) t -> bool ->
+    ('self, 'trec, 'a list) t
+  val opt : ('self, 'trec, 'a) t -> ('self, 'trec, 'a option) t
+  val self : ('self, mayrec, 'self) t
+  val next : ('self, mayrec, 'self) t
+  val token : 'c pattern -> ('self, norec, 'c) t
+  val rules : 'a Rules.t list -> ('self, norec, 'a) t
+
+end = struct
+
+  type ('self, 'trec, 'a) t = ('self, 'trec, 'a) ty_symbol
+  let nterm e = Snterm e
+  let nterml e l = Snterml (e, l)
+  let list0 s = Slist0 s
+  let list0sep s sep b = Slist0sep (s, sep, b)
+  let list1 s = Slist1 s
+  let list1sep s sep b = Slist1sep (s, sep, b)
+  let opt s = Sopt s
+  let self = Sself
+  let next = Snext
+  let token tok = Stoken tok
+  let rules (t : 'a Rules.t list) = srules t
+
+end and Rule : sig
+
+  type ('self, 'trec, 'f, 'r) t = ('self, 'trec, 'f, 'r) ty_rule
+
+  val stop : ('self, norec, 'r, 'r) t
+  val next :
+    ('self, _, 'a, 'r) t -> ('self, _, 'b) Symbol.t ->
+    ('self, mayrec, 'b -> 'a, 'r) t
+  val next_norec :
+    ('self, norec, 'a, 'r) Rule.t -> ('self, norec, 'b) Symbol.t ->
+    ('self, norec, 'b -> 'a, 'r) t
+
+end = struct
+
+  type ('self, 'trec, 'f, 'r) t = ('self, 'trec, 'f, 'r) ty_rule
+
+  let stop = TStop
+  let next r s = TNext (MayRec2, r, s)
+  let next_norec r s = TNext (NoRec2, r, s)
+
+end and Rules : sig
+
+  type 'a t = 'a ty_rules
+  val make : (_, norec, 'f, Loc.t -> 'a) Rule.t -> 'f -> 'a t
+
+end = struct
+
+  type 'a t = 'a ty_rules
+  let make p act = TRules (p, act)
+
+end
+
+module Production = struct
+
+  type 'a t = 'a ty_production
+  let make p act = TProd (p, act)
+
+end
+
+module Unsafe = struct
+
+  let clear_entry e =
+    e.estart <- (fun _ (strm__ : _ Stream.t) -> raise Stream.Failure);
+    e.econtinue <- (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
+    match e.edesc with
+      Dlevels _ -> e.edesc <- Dlevels []
+    | Dparser _ -> ()
+
+end
+
+type 'a single_extend_statement =
+  string option * Gramext.g_assoc option * 'a ty_production list
+
+type 'a extend_statement =
+  { pos : Gramext.position option
+  ; data : 'a single_extend_statement list
+  }
+
+let safe_extend (e : 'a Entry.t) { pos; data } =
+  extend_entry e pos data
+
+let safe_delete_rule e (TProd (r,_act)) =
+  let AnyS (symbols, _) = get_symbols r in
+  delete_rule e symbols
+
+let level_of_nonterm sym = match sym with
+  | Snterml (_,l) -> Some l
+  | _ -> None
+
+exception SelfSymbol
+
+let rec generalize_symbol :
+  type a tr s. (s, tr, a) Symbol.t -> (s, norec, a) ty_symbol =
+  function
+  | Stoken tok ->
+    Stoken tok
+  | Slist1 e ->
+    Slist1 (generalize_symbol e)
+  | Slist1sep (e, sep, b) ->
+    let e = generalize_symbol e in
+    let sep = generalize_symbol sep in
+    Slist1sep (e, sep, b)
+  | Slist0 e ->
+    Slist0 (generalize_symbol e)
+  | Slist0sep (e, sep, b) ->
+    let e = generalize_symbol e in
+    let sep = generalize_symbol sep in
+    Slist0sep (e, sep, b)
+  | Sopt e ->
+    Sopt (generalize_symbol e)
+  | Sself ->
+    raise SelfSymbol
+  | Snext ->
+    raise SelfSymbol
+  | Snterm e ->
+    Snterm e
+  | Snterml (e, l) ->
+    Snterml (e, l)
+  | Stree r ->
+    Stree (generalize_tree r)
+and generalize_tree : type a tr s .
+  (s, tr, a) ty_tree -> (s, norec, a) ty_tree = fun r ->
+  match r with
+  | Node (fi, n) ->
+    let fi = match fi with
+      | NoRec3 -> NoRec3
+      | MayRec3 -> raise SelfSymbol
+    in
+    let n = match n with
+      | { node; son; brother } ->
+        let node = generalize_symbol node in
+        let son = generalize_tree son in
+        let brother = generalize_tree brother in
+        { node; son; brother }
+    in
+    Node (fi, n)
+  | LocAct _ as r -> r
+  | DeadEnd as r -> r
+
+let generalize_symbol s =
+  try Some (generalize_symbol s)
+  with SelfSymbol -> None
+
+let rec mk_rule tok =
+  match tok with
+  | [] ->
+    let stop_e = Rule.stop in
+    TRules (stop_e, fun _ -> (* dropped anyway: *) "")
+  | tkn :: rem ->
+    let TRules (r, f) = mk_rule rem in
+    let r = Rule.next_norec r (Symbol.token tkn) in
+    TRules (r, fun _ -> f)
 
 end
