@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -133,10 +133,10 @@ and asks_for_coqtop args =
         ~filter:false
         ~filename:(coqtop_path ()) () in
       match file with
-      | Some _ ->
-          let () = custom_coqtop := file in
+      | [file] ->
+          let () = custom_coqtop := Some file in
           filter_coq_opts args
-      | None -> exit 0
+      | _ -> exit 0
 
 exception WrongExitStatus of string
 
@@ -558,6 +558,7 @@ struct
     { opts = [raw_matching]; init = true;
       label = "Display raw _matching expressions" };
     { opts = [notations]; init = true; label = "Display _notations" };
+    { opts = [notations]; init = true; label = "Display _parentheses" };
     { opts = [all_basic]; init = false;
       label = "Display _all basic low-level contents" };
     { opts = [existential]; init = false;
