@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -114,8 +114,10 @@ let make_coqtop_args fname =
         (* We basically copy the code of Names.check_valid since it is not exported *)
         (* to coqide. This is to prevent a possible failure of parsing  "-topfile"  *)
         (* at initialization of coqtop (see #10286) *)
+        (* If the file name is a valid identifier, use it as toplevel name; *)
+        (* otherwise the default “Top” will be used. *)
         match Unicode.ident_refutation (Filename.chop_extension (Filename.basename fname)) with
-        | Some (_,x) -> output_string stderr (x^"\n"); exit 1
+        | Some _ -> args
         | None -> "-topfile"::fname::args
   in
   proj, args
@@ -244,6 +246,13 @@ let close_and_quit () =
   List.iter (fun sn -> Coq.close_coqtop sn.coqtop) notebook#pages;
   exit 0
 
+(* Work around a deadlock due to OCaml exit cleanup. The standard [exit]
+   function calls [flush_all], which can block if one of the opened channels is
+   not valid anymore. We do not register [at_exit] functions in CoqIDE, so
+   instead of flushing we simply die as gracefully as possible in the function
+   below. *)
+external sys_exit : int -> 'a = "caml_sys_exit"
+
 let crash_save exitcode =
   Minilib.log "Starting emergency save of buffers in .crashcoqide files";
   let idx =
@@ -263,7 +272,7 @@ let crash_save exitcode =
   in
   List.iter save_session notebook#pages;
   Minilib.log "End emergency save";
-  exit exitcode
+  sys_exit exitcode
 
 end
 
@@ -282,9 +291,8 @@ let load ?parent _ =
   let filename =
     try notebook#current_term.fileops#filename
     with Invalid_argument _ -> None in
-  match select_file_for_open ~title:"Load file" ?parent ?filename () with
-    | None -> ()
-    | Some f -> FileAux.load_file f
+  let filenames = select_file_for_open ~title:"Load file" ~multiple:true ?parent ?filename () in
+  List.iter FileAux.load_file filenames
 
 let save ?parent _ = on_current_term (FileAux.check_save ?parent ~saveas:false)
 
@@ -461,7 +469,7 @@ let compile sn =
     |Some f ->
       let args = Coq.get_arguments sn.coqtop in
       let cmd = cmd_coqc#get
-        ^ " " ^ String.concat " " args
+        ^ " " ^ String.concat " " (List.map Filename.quote args)
         ^ " " ^ (Filename.quote f) ^ " 2>&1"
       in
       let buf = Buffer.create 1024 in
@@ -475,7 +483,7 @@ let compile sn =
           flash_info (f ^ " successfully compiled")
         else begin
           flash_info (f ^ " failed to compile");
-          sn.messages#default_route#set (Pp.str "Compilation output:\n");
+          sn.messages#default_route#set (Pp.str ("Compilation output:\n" ^ cmd ^ "\n"));
           sn.messages#default_route#add (Pp.str (Buffer.contents buf));
         end
       in
@@ -781,25 +789,28 @@ let coqtop_arguments sn =
   let dialog = GWindow.dialog ~title:"Coqtop arguments" () in
   let coqtop = sn.coqtop in
   (* Text entry *)
-  let args = Coq.get_arguments coqtop in
-  let text = String.concat " " args in
+  let text = Ideutils.encode_string_list (Coq.get_arguments coqtop) in
   let entry = GEdit.entry ~text ~packing:dialog#vbox#add () in
   (* Buttons *)
   let box = dialog#action_area in
   let ok = GButton.button ~stock:`OK ~packing:box#add () in
+  let fail s =
+    let msg = Printf.sprintf "Invalid arguments: %s" s in
+    let () = sn.messages#default_route#clear in
+    sn.messages#default_route#push Feedback.Error (Pp.str msg) in
   let ok_cb () =
-    let nargs = String.split_on_char ' ' entry#text in
-    if nargs <> args then
+    let ntext = entry#text in
+    if ntext <> text then
+      match try Util.Inr (Ideutils.decode_string_list ntext) with Failure s -> Util.Inl s with
+      | Util.Inl s -> fail s
+      | Util.Inr nargs ->
       let failed = Coq.filter_coq_opts nargs in
       match failed with
       | [] ->
         let () = Coq.set_arguments coqtop nargs in
         dialog#destroy ()
       | args ->
-        let args = String.concat " " args in
-        let msg = Printf.sprintf "Invalid arguments: %s" args in
-        let () = sn.messages#default_route#clear in
-        sn.messages#default_route#push Feedback.Error (Pp.str msg)
+        fail (String.concat " " args)
     else dialog#destroy ()
   in
   let _ = entry#connect#activate ~callback:ok_cb in
@@ -977,7 +988,6 @@ let build_ui () =
   let view_menu = GAction.action_group ~name:"View" () in
   let export_menu = GAction.action_group ~name:"Export" () in
   let navigation_menu = GAction.action_group ~name:"Navigation" () in
-  let tactics_menu = GAction.action_group ~name:"Tactics" () in
   let templates_menu = GAction.action_group ~name:"Templates" () in
   let tools_menu = GAction.action_group ~name:"Tools" () in
   let queries_menu = GAction.action_group ~name:"Queries" () in
@@ -985,7 +995,7 @@ let build_ui () =
   let windows_menu = GAction.action_group ~name:"Windows" () in
   let help_menu = GAction.action_group ~name:"Help" () in
   let all_menus = [
-    file_menu; edit_menu; view_menu; export_menu; navigation_menu; tactics_menu;
+    file_menu; edit_menu; view_menu; export_menu; navigation_menu;
     templates_menu; tools_menu; queries_menu; compile_menu; windows_menu;
     help_menu; ] in
 
@@ -996,8 +1006,6 @@ let build_ui () =
     item "Save" ~callback:(File.save ~parent:w) ~stock:`SAVE ~tooltip:"Save current buffer";
     item "Save as" ~label:"S_ave as" ~stock:`SAVE_AS ~callback:(File.saveas ~parent:w);
     item "Save all" ~label:"Sa_ve all" ~callback:File.saveall;
-    item "Revert all buffers" ~label:"_Revert all buffers"
-      ~callback:(File.revert_all ~parent:w) ~stock:`REVERT_TO_SAVED;
     item "Close buffer" ~label:"_Close buffer" ~stock:`CLOSE
       ~callback:(File.close_buffer ~parent:w) ~tooltip:"Close current buffer";
     item "Print..." ~label:"_Print..."
@@ -1121,11 +1129,6 @@ let build_ui () =
     ("Force", "_Force", `EXECUTE, Nav.join_document, "Fully check the document", "f");
   ] end;
 
-  menu tactics_menu [
-    item "Tactics" ~label:"_Tactics";
-  ];
-  alpha_items tactics_menu "Tactic" Coq_commands.tactics;
-
   menu templates_menu [
     item "Templates" ~label:"Te_mplates";
     template_item ("Lemma new_lemma : .\nProof.\n\nQed.\n", 6,9, "J");
@@ -1209,7 +1212,6 @@ let build_ui () =
   Coqide_ui.ui_m#insert_action_group edit_menu 0;
   Coqide_ui.ui_m#insert_action_group view_menu 0;
   Coqide_ui.ui_m#insert_action_group navigation_menu 0;
-  Coqide_ui.ui_m#insert_action_group tactics_menu 0;
   Coqide_ui.ui_m#insert_action_group templates_menu 0;
   Coqide_ui.ui_m#insert_action_group tools_menu 0;
   Coqide_ui.ui_m#insert_action_group queries_menu 0;
@@ -1293,7 +1295,10 @@ let build_ui () =
   (* Initializing hooks *)
   let refresh_style style =
     let style = style_manager#style_scheme style in
-    let iter_session v = v.script#source_buffer#set_style_scheme style in
+    let iter_session v =
+      v.script#source_buffer#set_style_scheme style;
+      v.proof#source_buffer#set_style_scheme style;
+      v.messages#default_route#source_buffer#set_style_scheme style in
     List.iter iter_session notebook#pages
   in
   let refresh_language lang =
@@ -1368,7 +1373,7 @@ let read_coqide_args argv =
     |"-debug"::args ->
       Minilib.debug := true;
       Flags.debug := true;
-      Backtrace.record_backtrace true;
+      Exninfo.record_backtrace true;
       filter_coqtop coqtop project_files bindings_files ("-debug"::out) args
     |"-coqtop-flags" :: flags :: args->
       Coq.ideslave_coqtop_flags := Some flags;
