@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -49,6 +49,7 @@ let coqide_known_option table = List.mem table [
   ["Printing";"Matching"];
   ["Printing";"Synth"];
   ["Printing";"Notations"];
+  ["Printing";"Parentheses"];
   ["Printing";"All"];
   ["Printing";"Records"];
   ["Printing";"Existential";"Instances"];
@@ -68,9 +69,9 @@ let ide_cmd_checks ~last_valid { CAst.loc; v } =
   let user_error s =
     try CErrors.user_err ?loc ~hdr:"IDE" (str s)
     with e ->
-      let (e, info) = CErrors.push e in
+      let (e, info) = Exninfo.capture e in
       let info = Stateid.add info ~valid:last_valid Stateid.dummy in
-      Exninfo.raise ~info e
+      Exninfo.iraise (e, info)
   in
   if is_debug v.expr then
     user_error "Debug mode not available in the IDE"
@@ -196,7 +197,7 @@ let process_goal sigma g =
   let min_env = Environ.reset_context env in
   let id = Goal.uid g in
   let ccl =
-    pr_goal_concl_style_env env sigma (Goal.V82.concl sigma g)
+    pr_letype_env ~goal_concl_style:true env sigma (Goal.V82.concl sigma g)
   in
   let process_hyp d (env,l) =
     let d' = CompactedDecl.to_named_context d in
@@ -231,32 +232,32 @@ let goals () =
   let doc = get_doc () in
   set_doc @@ Stm.finish ~doc;
   try
-    let newp = Vernacstate.Proof_global.give_me_the_proof () in
+    let newp = Vernacstate.Declare.give_me_the_proof () in
     if Proof_diffs.show_diffs () then begin
       let oldp = Stm.get_prev_proof ~doc (Stm.get_current_state ~doc) in
       let diff_goal_map = Proof_diffs.make_goal_map oldp newp in
       Some (export_pre_goals Proof.(data newp) (process_goal_diffs diff_goal_map oldp))
     end else
       Some (export_pre_goals Proof.(data newp) process_goal)
-  with Vernacstate.Proof_global.NoCurrentProof -> None
+  with Vernacstate.Declare.NoCurrentProof -> None
   [@@ocaml.warning "-3"];;
 
 let evars () =
   try
     let doc = get_doc () in
     set_doc @@ Stm.finish ~doc;
-    let pfts = Vernacstate.Proof_global.give_me_the_proof () in
+    let pfts = Vernacstate.Declare.give_me_the_proof () in
     let Proof.{ sigma } = Proof.data pfts in
     let exl = Evar.Map.bindings (Evd.undefined_map sigma) in
     let map_evar ev = { Interface.evar_info = string_of_ppcmds (pr_evar sigma ev); } in
     let el = List.map map_evar exl in
     Some el
-  with Vernacstate.Proof_global.NoCurrentProof -> None
+  with Vernacstate.Declare.NoCurrentProof -> None
   [@@ocaml.warning "-3"]
 
 let hints () =
   try
-    let pfts = Vernacstate.Proof_global.give_me_the_proof () in
+    let pfts = Vernacstate.Declare.give_me_the_proof () in
     let Proof.{ goals; sigma } = Proof.data pfts in
     match goals with
     | [] -> None
@@ -265,7 +266,7 @@ let hints () =
       let get_hint_hyp env d accu = hyp_next_tac sigma env d :: accu in
       let hint_hyps = List.rev (Environ.fold_named_context get_hint_hyp env ~init: []) in
       Some (hint_hyps, concl_next_tac)
-  with Vernacstate.Proof_global.NoCurrentProof -> None
+  with Vernacstate.Declare.NoCurrentProof -> None
   [@@ocaml.warning "-3"]
 
 (** Other API calls *)
@@ -286,11 +287,11 @@ let status force =
     List.rev_map Names.Id.to_string l
   in
   let proof =
-    try Some (Names.Id.to_string (Vernacstate.Proof_global.get_current_proof_name ()))
-    with Vernacstate.Proof_global.NoCurrentProof -> None
+    try Some (Names.Id.to_string (Vernacstate.Declare.get_current_proof_name ()))
+    with Vernacstate.Declare.NoCurrentProof -> None
   in
   let allproofs =
-    let l = Vernacstate.Proof_global.get_all_proof_names () in
+    let l = Vernacstate.Declare.get_all_proof_names () in
     List.map Names.Id.to_string l
   in
   {
@@ -339,8 +340,11 @@ let import_search_constraint = function
   | Interface.Include_Blacklist -> Search.Include_Blacklist
 
 let search flags =
-  let pstate = Vernacstate.Proof_global.get_pstate () in
-  List.map export_coq_object (Search.interface_search ?pstate (
+  let pstate = Vernacstate.Declare.get_pstate () in
+  let sigma, env = match pstate with
+  | None -> let env = Global.env () in Evd.(from_env env, env)
+  | Some p -> Declare.get_goal_context p 1 in
+  List.map export_coq_object (Search.interface_search env sigma (
     List.map (fun (c, b) -> (import_search_constraint c, b)) flags)
   )
   [@@ocaml.warning "-3"]
@@ -360,7 +364,6 @@ let import_option_value = function
 let export_option_state s = {
   Interface.opt_sync  = true;
   Interface.opt_depr  = s.Goptions.opt_depr;
-  Interface.opt_name  = s.Goptions.opt_name;
   Interface.opt_value = export_option_value s.Goptions.opt_value;
 }
 
@@ -477,7 +480,7 @@ let print_xml =
   fun oc xml ->
     Mutex.lock m;
     try Control.protect_sigalrm (Xml_printer.print oc) xml; Mutex.unlock m
-    with e -> let e = CErrors.push e in Mutex.unlock m; iraise e
+    with e -> let e = Exninfo.capture e in Mutex.unlock m; Exninfo.iraise e
 
 let slave_feeder fmt xml_oc msg =
   let xml = Xmlprotocol.(of_feedback fmt msg) in
