@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -19,10 +19,20 @@ type universe_decl_expr = (lident list, Glob_term.glob_constraint list) UState.g
 type ident_decl = lident * universe_decl_expr option
 type name_decl = lname * universe_decl_expr option
 
+type notation_with_optional_scope = LastLonelyNotation | NotationInScope of string
+
+type entry_level = int
+type entry_relative_level = LevelLt of entry_level | LevelLe of entry_level | LevelSome
+
 type notation_entry = InConstrEntry | InCustomEntry of string
-type notation_entry_level = InConstrEntrySomeLevel | InCustomEntryLevel of string * int
+type notation_entry_level = InConstrEntrySomeLevel | InCustomEntryLevel of string * entry_level
 type notation_key = string
-type notation = notation_entry_level * notation_key
+
+(* A notation associated to a given parsing rule *)
+type notation = notation_entry * notation_key
+
+(* A notation associated to a given interpretation *)
+type specific_notation = notation_with_optional_scope * (notation_entry * notation_key)
 
 type 'a or_by_notation_r =
   | AN of 'a
@@ -47,26 +57,8 @@ type abstraction_kind = AbsLambda | AbsPi
 
 type proj_flag = int option (** [Some n] = proj of the n-th visible argument *)
 
-(** Representation of decimal literals that appear in Coq scripts.
-   We now use raw strings following the format defined by
-   [NumTok.t] and a separate sign flag.
-
-   Note that this representation is not unique, due to possible
-   multiple leading or trailing zeros, and -0 = +0, for instances.
-   The reason to keep the numeral exactly as it was parsed is that
-   specific notations can be declared for specific numerals
-   (e.g. [Notation "0" := False], or [Notation "00" := (nil,nil)], or
-   [Notation "2e1" := ...]). Those notations, which override the
-   generic interpretation as numeral, use the same representation of
-   numeral using the Numeral constructor. So the latter should be able
-   to record the form of the numeral which exactly matches the
-   notation. *)
-
-type sign = SPlus | SMinus
-type raw_numeral = NumTok.t
-
 type prim_token =
-  | Numeral of sign * raw_numeral
+  | Numeral of NumTok.Signed.t
   | String of string
 
 type instance_expr = Glob_term.glob_level list
@@ -78,7 +70,7 @@ type cases_pattern_expr_r =
   (** [CPatCstr (_, c, Some l1, l2)] represents [(@ c l1) l2] *)
   | CPatAtom of qualid option
   | CPatOr   of cases_pattern_expr list
-  | CPatNotation of notation * cases_pattern_notation_substitution
+  | CPatNotation of notation_with_optional_scope option * notation * cases_pattern_notation_substitution
     * cases_pattern_expr list (** CPatNotation (_, n, l1 ,l2) represents
                                   (notation n applied with substitution l1)
                                   applied to arguments l2 *)
@@ -119,7 +111,7 @@ and constr_expr_r =
   | CEvar   of Glob_term.existential_name * (Id.t * constr_expr) list
   | CSort   of Glob_term.glob_sort
   | CCast   of constr_expr * constr_expr Glob_term.cast_type
-  | CNotation of notation * constr_notation_substitution
+  | CNotation of notation_with_optional_scope option * notation * constr_notation_substitution
   | CGeneralization of Glob_term.binding_kind * abstraction_kind option * constr_expr
   | CPrim of prim_token
   | CDelimiters of string * constr_expr
