@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -67,7 +67,7 @@ end
 type stratification = {
   env_universes : UGraph.t;
   env_sprop_allowed : bool;
-  env_universes_lbound : Univ.Level.t;
+  env_universes_lbound : UGraph.Bound.t;
   env_engagement : engagement
 }
 
@@ -128,8 +128,8 @@ let empty_env = {
   env_nb_rel = 0;
   env_stratification = {
     env_universes = UGraph.initial_universes;
-    env_sprop_allowed = false;
-    env_universes_lbound = Univ.Level.set;
+    env_sprop_allowed = true;
+    env_universes_lbound = UGraph.Bound.Set;
     env_engagement = PredicativeSet };
   env_typing_flags = Declareops.safe_flags Conv_oracle.empty;
   retroknowledge = Retroknowledge.empty;
@@ -275,11 +275,13 @@ let type_in_type env = not (typing_flags env).check_universes
 let deactivated_guard env = not (typing_flags env).check_guarded
 
 let indices_matter env = env.env_typing_flags.indices_matter
-let check_template env = env.env_typing_flags.check_template
 
 let universes env = env.env_stratification.env_universes
 let universes_lbound env = env.env_stratification.env_universes_lbound
 
+let set_universes g env =
+  {env with env_stratification = {env.env_stratification with env_universes=g}}
+
 let set_universes_lbound env lbound =
   let env_stratification = { env.env_stratification with env_universes_lbound = lbound } in
   { env with env_stratification }
@@ -399,9 +401,6 @@ let add_constraints c env =
 let check_constraints c env =
   UGraph.check_constraints c env.env_stratification.env_universes
 
-let push_constraints_to_env (_,univs) env =
-  add_constraints univs env
-
 let add_universes ~lbound ~strict ctx g =
   let g = Array.fold_left
             (fun g v -> UGraph.add_universe ~lbound ~strict v g)
@@ -435,7 +434,7 @@ let push_subgraph (levels,csts) env =
   in
   map_universes add_subgraph env
 
-let set_engagement c env = (* Unsafe *)
+let set_engagement c env =
   { env with env_stratification =
     { env.env_stratification with env_engagement = c } }
 
@@ -449,7 +448,7 @@ let same_flags {
      share_reduction;
      enable_VM;
      enable_native_compiler;
-     check_template;
+     cumulative_sprop;
   } alt =
   check_guarded == alt.check_guarded &&
   check_positive == alt.check_positive &&
@@ -459,14 +458,17 @@ let same_flags {
   share_reduction == alt.share_reduction &&
   enable_VM == alt.enable_VM &&
   enable_native_compiler == alt.enable_native_compiler &&
-  check_template == alt.check_template
+  cumulative_sprop == alt.cumulative_sprop
 [@warning "+9"]
 
-let set_typing_flags c env = (* Unsafe *)
-  if same_flags env.env_typing_flags c then env
-  else { env with env_typing_flags = c }
+let set_cumulative_sprop b = map_universes (UGraph.set_cumulative_sprop b)
 
-let make_sprop_cumulative = map_universes UGraph.make_sprop_cumulative
+let set_typing_flags c env =
+  if same_flags env.env_typing_flags c then env
+  else set_cumulative_sprop c.cumulative_sprop { env with env_typing_flags = c }
+
+let set_cumulative_sprop b env =
+  set_typing_flags {env.env_typing_flags with cumulative_sprop=b} env
 
 let set_allow_sprop b env =
   { env with env_stratification =
@@ -591,19 +593,16 @@ let polymorphic_pind (ind,u) env =
 let type_in_type_ind (mind,_i) env =
   not (lookup_mind mind env).mind_typing_flags.check_universes
 
-let template_checked_ind (mind,_i) env =
-  (lookup_mind mind env).mind_typing_flags.check_template
-
 let template_polymorphic_ind (mind,i) env =
   match (lookup_mind mind env).mind_packets.(i).mind_arity with
   | TemplateArity _ -> true
   | RegularArity _ -> false
 
-let template_polymorphic_variables (mind,i) env =
-  match (lookup_mind mind env).mind_packets.(i).mind_arity with
-  | TemplateArity { Declarations.template_param_levels = l; _ } ->
+let template_polymorphic_variables (mind, _) env =
+  match (lookup_mind mind env).mind_template with
+  | Some { Declarations.template_param_levels = l; _ } ->
     List.map_filter (fun level -> level) l
-  | RegularArity _ -> []
+  | None -> []
 
 let template_polymorphic_pind (ind,u) env =
   if not (Univ.Instance.is_empty u) then false
@@ -802,14 +801,6 @@ let get_template_polymorphic_variables env r =
   | IndRef ind -> template_polymorphic_variables ind env
   | ConstructRef cstr -> template_polymorphic_variables (inductive_of_constructor cstr) env
 
-let is_template_checked env r =
-  let open Names.GlobRef in
-  match r with
-  | VarRef _id -> false
-  | ConstRef _c -> false
-  | IndRef ind -> template_checked_ind ind env
-  | ConstructRef cstr -> template_checked_ind (inductive_of_constructor cstr) env
-
 let is_type_in_type env r =
   let open Names.GlobRef in
   match r with
