@@ -1,7 +1,7 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
-(* <O___,, *       (see CREDITS file for the list of authors)           *)
+(*  v      *         Copyright INRIA, CNRS and contributors             *)
+(* <O___,, * (see version control and CREDITS file for authors & dates) *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
 (*         *     GNU Lesser General Public License Version 2.1          *)
@@ -225,8 +225,7 @@ and subst_body : 'a. _ -> _ -> (_ -> 'a -> 'a) -> _ -> 'a generic_module_body ->
      && retro==retro' && delta'==mb.mod_delta
   then mb
   else
-    { mb with
-      mod_mp = mp';
+    { mod_mp = mp';
       mod_expr = me';
       mod_type = ty';
       mod_type_alg = aty';
@@ -382,21 +381,21 @@ let inline_delta_resolver env inl mp mbid mtb delta =
   let rec make_inline delta = function
     | [] -> delta
     | (lev,kn)::r ->
-        let kn = replace_mp_in_kn (MPbound mbid) mp kn in
-        let con = constant_of_delta_kn delta kn in
-        try
-          let constant = lookup_constant con env in
-          let l = make_inline delta r in
-          match constant.const_body with
-            | Undef _ | OpaqueDef _ | Primitive _ -> l
-            | Def body ->
-              let constr = Mod_subst.force_constr body in
-              let ctx = Declareops.constant_polymorphic_context constant in
-              let constr = Univ.{univ_abstracted_value=constr; univ_abstracted_binder=ctx} in
-              add_inline_delta_resolver kn (lev, Some constr) l
-        with Not_found ->
-          error_no_such_label_sub (Constant.label con)
-            (ModPath.to_string (Constant.modpath con))
+      let kn = replace_mp_in_kn (MPbound mbid) mp kn in
+      let con = constant_of_delta_kn delta kn in
+      if not (Environ.mem_constant con env) then
+        error_no_such_label_sub (Constant.label con)
+          (ModPath.to_string (Constant.modpath con))
+      else
+        let constant = lookup_constant con env in
+        let l = make_inline delta r in
+        match constant.const_body with
+        | Undef _ | OpaqueDef _ | Primitive _ -> l
+        | Def body ->
+          let constr = Mod_subst.force_constr body in
+          let ctx = Declareops.constant_polymorphic_context constant in
+          let constr = Univ.{univ_abstracted_value=constr; univ_abstracted_binder=ctx} in
+          add_inline_delta_resolver kn (lev, Some constr) l
   in
   make_inline delta constants
 
